---
-- This is the main module of HsUnix. It is the only module meant to be
-- used directly by applications. The module itself copes with naming
-- stuff. It imports and exports the HsUnix modules, as well as some system
-- modules. Most scripts should not need to to import anything besides
-- <code>HsUnix</code>. 
-- @version 0.9
module HsUnix (
              -- * Command Line Arguments
                module HsUnixArgs
              -- * Specific External Commands
              , module HsUnixCommands
              -- * Miscellaneous
              , module HsUnixMisc
              -- * Parsing and Composing Paths
              , module HsUnixPaths
              , module HsUnixProcErr
              , mainwrapper
              -- * Calling a Shell, and Quoting for Shells
              , module HsUnixShell

              -- * Reexported Standard Library Stuff for Exception Handling
              , Exception.Exception (..)
              , Exception.IOException
              , Exception.ArithException (..)
              , Exception.ArrayException (..)
              , Exception.AsyncException (..)
              , Exception.throw
              , Exception.ioError
              , Exception.throwTo
              , Exception.catchJust
              , Exception.handle
              , Exception.handleJust
              , Exception.tryJust
              , Exception.evaluate
              , Exception.ioErrors
              , Exception.arithExceptions
              , Exception.errorCalls
              , Exception.dynExceptions
              , Exception.assertions
              , Exception.asyncExceptions
              , Exception.userErrors
              , Exception.throwDyn
              , Exception.throwDynTo
              , Exception.catchDyn
              , Exception.block
              , Exception.unblock
              , Exception.assert
              , Exception.finally
              , Dynamic.toDyn
              , Dynamic.fromDyn
              , Dynamic.fromDynamic
              , Dynamic.dynApply
              , Dynamic.dynApp
              , Dynamic.Typeable (..)
              , Dynamic.TypeRep
              , Dynamic.TyCon
              , Dynamic.mkTyCon
              , Dynamic.mkAppTy
              , Dynamic.mkFunTy
              , Dynamic.applyTy
              , Directory.Permissions (..)
              , Directory.createDirectory
              , Directory.removeDirectory
              , Directory.renameDirectory
              , Directory.getDirectoryContents
              , Directory.getCurrentDirectory
              , Directory.setCurrentDirectory
              , Directory.removeFile
              , Directory.renameFile
              , Directory.doesFileExist
              , Directory.doesDirectoryExist
              , Directory.getPermissions
              , Directory.setPermissions
              , Directory.getModificationTime
              , Posix.fileAccess
              )
where

import System
import List
import Monad
import GetOpt
import Maybe
import IO
import Random
import HsUnixArgs
import HsUnixCommands
import HsUnixMisc
import HsUnixPaths
import HsUnixProcErr
import HsUnixShell
import Exception
import Dynamic
import Directory
import Posix
import GHC.IOBase


{- | Error reporting wrapper for the @main@ function. This catches any
   HsUnix generated exception, and @IOError@s from the @IO@ library. prints
   an error message and exits with @exitFailure@. The @main@ function
   typically looks like this:

                         >main = mainwrapper main'

   The exceptions caught are @ArgError@, @Errno@, @ProcessStatus@, @IOError@.
-}
mainwrapper :: IO a     -- ^ Should be @main@
            -> IO a     -- ^ Wrapped @main@
mainwrapper io =
    io
    `catchDyn`
       (\argerror -> do
           hPutStrLn stderr $ (argerror_message argerror) ++ "\n\n" ++ (argerror_usageinfo argerror)
           exitFailure
       )
    `catchDyn`
       (\processstatus -> do
           hPutStrLn stderr $ "Process error. process status = " ++ show ( processstatus :: ProcessStatus )
           exitFailure
       )
    `IO.catch`
       (\ioe -> do
           hPutStrLn stderr (show_ioerror ioe)
           exitFailure
       )

